/*!
 * @file        apm32f0xx_pmu.h
 *
 * @brief       This file contains all functions prototype and macros for the PMU peripheral
 *
 * @version     V1.0.0
 *
 * @date        2019-10-16
 *
 */

#ifndef __PMU_H
#define __PMU_H

#include "apm32f0xx.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup PMU_Driver  PMU Driver
  @{
*/

/** @addtogroup PMU_Enumerations Enumerations
  @{
*/

/**
 * @brief   AHB1 peripheral
 */
typedef enum
{
    PMU_WAKEUPPIN_1 = 0x00,
    PMU_WAKEUPPIN_2 = 0x01,
} PMU_WAKEUPPIN_T;

/**
 * @brief   SLEEP_mode_entry
 */
typedef enum
{
    PMU_SLEEPENTRY_WFI = 0x00,
    PMU_SLEEPENTRY_WFE = 0x01,
} PMU_SLEEPENTRY_T;

/**
 * @brief   Regulator_state_is_Sleep_STOP_mode
 */
typedef enum
{
    PMU_REGULATOR_ON = 0x00,
    PMU_REGULATOR_LowPower = 0x01,
} PMU_REGULATOR_T;

/**
 * @brief   STOP_mode_entry
 */
typedef enum
{
    PMU_STOPENTRY_WFI = 0x00,
    PMU_STOPENTRY_WFE = 0x01,
    PMU_STOPENTRY_SLEEPONEXIT = 0x02,
} PMU_STOPENTRY_T;

/**
 * @brief   Flag
 */
typedef enum
{
    PMU_FLAG_WUPF = 0x01,
    PMU_FLAG_STDBYF = 0x02,
} PMU_FLAG_T;

/**@} end of group PMU_Enumerations*/


/** @addtogroup PMU_Fuctions Fuctions
  @{
*/

/** Function used to set the PMU configuration to the default reset state */
void PMU_Reset(void);

/** Backup Domain Access function */
void PMU_EnableBackupAccess(void);
void PMU_DisableBackupAccess(void);

/** WakeUp pins configuration functions */
void PMU_EnableWakeUpPin(PMU_WAKEUPPIN_T pin);
void PMU_DisableWakeUpPin(PMU_WAKEUPPIN_T pin);

/** Low Power modes configuration functions */
void PMU_EnterSleepMode(PMU_SLEEPENTRY_T entry);
void PMU_EnterSTOPMode(PMU_REGULATOR_T regulator, PMU_STOPENTRY_T entry);
void PMU_EnterSTANDBYMode(void);

/** Flags management functions */
uint8_t PMU_ReadStatusFlag(PMU_FLAG_T flag);
void PMU_ClearStatusFlag(uint8_t flag);

/**@} end of group PMU_Fuctions*/
/**@} end of group PMU_Driver */
/**@} end of group Peripherals_Library*/

#ifdef __cplusplus
}
#endif

#endif /* __PMU_H */

