/*!
 * @file        apm32f0xx_rcm.h
 * @brief       This file contains all the functions prototypes for the RCM firmware library
 *
 * @details
 *
 * @version     V1.0.0
 * @author
 * @date        2020-5-14
 *
 */

#ifndef __RCM_H
#define __RCM_H

#include "apm32f0xx.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup RCM_Driver RCM Driver
  @{
*/

/** @addtogroup RCM_Enumerations Enumerations
  @{
*/

/**
 * @brief   HXT enum
 */
typedef enum
{
    RCM_HXT_CLOSE =0x00, ///!< CLOSE HXT
    RCM_HXT_OPEN  =0x01, ///!< OPEN HXT
    RCM_HXT_BYPASS=0x05, ///!< HXT BYPASS
} RCM_HXT_T;

/**
 * @brief   LXT enum
 */
typedef enum
{
    RCM_LXT_CLOSE =0x00, ///!< CLOSE HXT
    RCM_LXT_OPEN  =0x01, ///!< OPEN HXT
    RCM_LXT_BYPASS=0x05, ///!< HXT BYPASS
} RCM_LXT_T;

/**
 * @brief   LXT_Drive enum
 */
typedef enum
{
    RCM_LXT_DRIVE_Low =0x00,
    RCM_LXT_DRIVE_MediumLow,
    RCM_LXT_DRIVE_MediumHigh,
    RCM_LXT_DRIVE_High,
} RCM_LXT_DRIVE_T;

/**
 * @brief   System clock select
 */
typedef enum
{
    RCM_PLL_SEL_HIRC_DIV2,
    RCM_PLL_SEL_PREDIV1,
} RCM_PLL_SEL_T;

/**
 * @brief   PLL multiplication factor
 */
typedef enum
{
    RCM_PLLMF_2,
    RCM_PLLMF_3,
    RCM_PLLMF_4,
    RCM_PLLMF_5,
    RCM_PLLMF_6,
    RCM_PLLMF_7,
    RCM_PLLMF_8,
    RCM_PLLMF_9,
    RCM_PLLMF_10,
    RCM_PLLMF_11,
    RCM_PLLMF_12,
    RCM_PLLMF_13,
    RCM_PLLMF_14,
    RCM_PLLMF_15,
    RCM_PLLMF_16,
} RCM_PLLMF_T;

/**
 * @brief   RCM_CLK_division_factor
 */
typedef enum
{
    RCM_CLK_Div1,
    RCM_CLK_Div2,
    RCM_CLK_Div3,
    RCM_CLK_Div4,
    RCM_CLK_Div5,
    RCM_CLK_Div6,
    RCM_CLK_Div7,
    RCM_CLK_Div8,
    RCM_CLK_Div9,
    RCM_CLK_Div10,
    RCM_CLK_Div11,
    RCM_CLK_Div12,
    RCM_CLK_Div13,
    RCM_CLK_Div14,
    RCM_CLK_Div15,
    RCM_CLK_Div16,
} RCM_CLK_Div_T;

/**
 * @brief   Clock output control
 */
typedef enum
{
    RCM_COC_NO_CLOCK =0x00,
    RCM_COC_HIRC14,
    RCM_COC_LIRC,
    RCM_COC_LXT,
    RCM_COC_SYSCLK,
    RCM_COC_HIRC,
    RCM_COC_HXT,
    RCM_COC_PLLCLK_DIV_2,
    RCM_COC_HIRC48,
    RCM_COC_PLLCLK =0x87,
} RCM_COCCLK_T;

/**
 * @brief   Clock output control
 */
typedef enum
{
    RCM_COC_PRE_1 =0x00,
    RCM_COC_PRE_2,
    RCM_COC_PRE_4,
    RCM_COC_PRE_8,
    RCM_COC_PRE_16,
    RCM_COC_PRE_32,
    RCM_COC_PRE_64,
    RCM_COC_PRE_128,
} RCM_COCPRE_T;

/**
 * @brief   System clock select
 */
typedef enum
{
    RCM_SYSCLK_SEL_HIRC,
    RCM_SYSCLK_SEL_HXT,
    RCM_SYSCLK_SEL_PLL,
} RCM_SYSCLK_SEL_T;

/**
 * @brief   AHB divider Number
 */
typedef enum
{
    RCM_SYSCLK_DIV_1 = 7,
    RCM_SYSCLK_DIV_2,
    RCM_SYSCLK_DIV_4,
    RCM_SYSCLK_DIV_8,
    RCM_SYSCLK_DIV_16,
    RCM_SYSCLK_DIV_64,
    RCM_SYSCLK_DIV_128,
    RCM_SYSCLK_DIV_256,
    RCM_SYSCLK_DIV_512,
} RCM_AHBDIV_T;

/**
 * @brief   APB divider Number
 */
typedef enum
{
    RCM_HCLK_DIV_1 = 3,
    RCM_HCLK_DIV_2,
    RCM_HCLK_DIV_4,
    RCM_HCLK_DIV_8,
    RCM_HCLK_DIV_16
} RCM_APBDIV_T;

/**
 * @brief   CEC divider Number
 */
typedef enum
{
    RCM_CECCLK_HIRC_DIV_224,
    RCM_CECCLK_LIRC_DIV,
} RCM_CECDIV_T;

/**
 * @brief   I2C divider Number
 */
typedef enum
{
    RCM_I2C1CLK_HIRC,
    RCM_I2C1CLK_SYSCLK,
} RCM_I2CDIV_T;

/**
 * @brief   USART divider Number
 */
typedef enum
{
    RCM_USARTCLK_PCLK,
    RCM_USARTCLK_SYSCLK,
    RCM_USARTCLK_LXT,
    RCM_USARTCLK_HIRC,
} RCM_USARTDIV_T;

/**
 * @brief   RTC clock select
 */
typedef enum
{
    RCM_RTCCLK_SEL_LXT = 0X01,
    RCM_RTCCLK_SEL_LIRC,
    RCM_RTCCLK_SEL_HXT_DIV_32
} RCM_RTCCLK_SEL_T;

/**
 * @brief   AHB peripheral
 */
typedef enum
{
    RCM_AHB_PERIPH_DMA1     = BIT0,
    RCM_AHB_PERIPH_SRAM     = BIT2,
    RCM_AHB_PERIPH_FPU      = BIT4,
    RCM_AHB_PERIPH_CRC      = BIT6,
    RCM_AHB_PERIPH_GPIOA    = BIT17,
    RCM_AHB_PERIPH_GPIOB    = BIT18,
    RCM_AHB_PERIPH_GPIOC    = BIT19,
    RCM_AHB_PERIPH_GPIOD    = BIT20,
    RCM_AHB_PERIPH_GPIOF    = BIT22,
} RCM_AHB_PERIPH_T;

/**
 * @brief   AHB2 peripheral
 */
typedef enum
{
    RCM_APB2_PERIPH_SYSCFG  = BIT0,
    RCM_APB2_PERIPH_USART6  = BIT5,
    RCM_APB2_PERIPH_ADC1    = BIT9,
    RCM_APB2_PERIPH_TMR1    = BIT11,
    RCM_APB2_PERIPH_SPI1    = BIT12,
    RCM_APB2_PERIPH_USART1  = BIT14,
    RCM_APB2_PERIPH_TMR15   = BIT16,
    RCM_APB2_PERIPH_TMR16   = BIT17,
    RCM_APB2_PERIPH_TMR17   = BIT18,
    RCM_APB2_PERIPH_DBGMCU  = BIT22,
} RCM_APB2_PERIPH_T;

/**
 * @brief   AHB1 peripheral
 */
typedef enum
{
    RCM_APB1_PERIPH_TMR3    = BIT1,
    RCM_APB1_PERIPH_TMR6    = BIT4,
    RCM_APB1_PERIPH_TMR7    = BIT5,
    RCM_APB1_PERIPH_TMR14   = BIT8,
    RCM_APB1_PERIPH_WWDT    = BIT11,
    RCM_APB1_PERIPH_SPI2    = BIT14,
    RCM_APB1_PERIPH_USART2  = BIT17,
    RCM_APB1_PERIPH_USART3  = BIT18,
    RCM_APB1_PERIPH_USART4  = BIT19,
    RCM_APB1_PERIPH_USART5  = BIT20,
    RCM_APB1_PERIPH_I2C1    = BIT21,
    RCM_APB1_PERIPH_I2C2    = BIT22,
    RCM_APB1_PERIPH_USB     = BIT23,
    RCM_APB1_PERIPH_PMU     = BIT28,
} RCM_APB1_PERIPH_T;

/**
 * @brief   RCM Interrupt Source
 */
typedef enum
{
    RCM_INT_LIRCRDY     = BIT0,             ///!<   LIRC ready interrupt
    RCM_INT_LXTRDY      = BIT1,             ///!<   LXT ready interrupt
    RCM_INT_HIRCRDY     = BIT2,             ///!<   HIRC ready interrupt
    RCM_INT_HXTRDY      = BIT3,             ///!<   HXT ready interrupt
    RCM_INT_PLLRDY      = BIT4,             ///!<   PLL ready interrupt
    RCM_INT_HIRC14RDY   = BIT5,             ///!<   HIRC14 ready interrupt
    RCM_INT_CSS         = BIT7              ///!<   Clock security system interrupt
} RCM_INT_T;

/**
 * @brief   RCM FLAG define
 */
typedef enum
{
    RCM_FLAG_HIRCRDY    = 0x001,        ///!<   HIRC Ready Flag
    RCM_FLAG_HXTRDY     = 0x011,        ///!<   HXT Ready Flag
    RCM_FLAG_PLLRDY     = 0x019,        ///!<   PLL Ready Flag
    RCM_FLAG_LXTRDY     = 0x101,        ///!<   LXT Ready Flag
    RCM_FLAG_LIRCRDY    = 0x201,        ///!<   LIRC Ready Flag
    RCM_FLAG_V18PRRST   = 0x217,        ///!<   Software reset flag
    RCM_FLAG_OBRST      = 0x219,        ///!<   Option byte loader reset flag
    RCM_FLAG_PINRST     = 0x21A,        ///!<   PIN reset flag
    RCM_FLAG_PWRRST     = 0x21B,        ///!<   POR/PDR reset flag
    RCM_FLAG_SWRST      = 0x21C,        ///!<   Software reset flag
    RCM_FLAG_IWDTRST    = 0x21D,        ///!<   Independent watchdog reset flag
    RCM_FLAG_WWDTRST    = 0x21E,        ///!<   Window watchdog reset flag
    RCM_FLAG_LPRRST     = 0x21F,        ///!<   Low-power reset flag
} RCM_FLAG_T;

/**@} end of group RCM_Enumerations*/


/** @addtogroup RCM_Fuctions Fuctions
  @{
*/

/** Function description */

void RCM_Reset(void);

void RCM_ConfigHXT(RCM_HXT_T state);
uint8_t RCM_WaitHXTReady(void);
void RCM_SetHIRCTrim(uint8_t HIRCTrim);
void RCM_EnableHIRC(void);
void RCM_DisableHIRC(void);

void RCM_SetHIRC14Trim(uint8_t HIRC14Trim);
void RCM_EnableHIRC14(void);
void RCM_DisableHIRC14(void);
void RCM_EnableHIRC14ADC(void);
void RCM_DisableHIRC14ADC(void);

void RCM_ConfigLXT(RCM_LXT_T state);
void RCM_ConfigDriveLXT(RCM_LXT_DRIVE_T state);

void RCM_EnableLIRC(void);
void RCM_DisableLIRC(void);

void RCM_ConfigPLL(RCM_PLL_SEL_T pllSelect, RCM_PLLMF_T pllMf);
void RCM_EnablePLL(void);
void RCM_DisablePLL(void);

void RCM_ConfigCLKDIV(RCM_CLK_Div_T state);

void RCM_EnableCCS(void);
void RCM_DisableCCS(void);

void RCM_ConfigCOC(RCM_COCCLK_T cocClock, RCM_COCPRE_T divided);
void RCM_ConfigSYSCLK(RCM_SYSCLK_SEL_T sysClkSelect);
RCM_SYSCLK_SEL_T RCM_ReadSYSCLKSource(void);

void RCM_ConfigAHB(RCM_AHBDIV_T AHBDiv);
void RCM_ConfigAPB1(RCM_APBDIV_T APBDiv);
void RCM_ConfigCECCLK(RCM_CECDIV_T CECDiv);
void RCM_ConfigI2CCLK(RCM_I2CDIV_T I2CDiv);
void RCM_ConfigUSARTCLK(RCM_USARTDIV_T USARTDiv);

uint32_t RCM_ReadSYSCLKFreq(void);
uint32_t RCM_ReadHCLKFreq(void);
uint32_t RCM_ReadPCLKFreq(void);
uint32_t RCM_ReadADCCLKFreq(void);
uint32_t RCM_ReadCECCLKFreq(void);
uint32_t RCM_ReadI2C1CLKFreq(void);
uint32_t RCM_ReadUSART1CLKFreq(void);
uint32_t RCM_ReadUSART2CLKFreq(void);

void RCM_ConfigRTCCLK(RCM_RTCCLK_SEL_T rtcClkSelect);
void RCM_EnableRTCCLK(void);
void RCM_DisableRTCCLK(void);

void RCM_EnableBackupReset(void);
void RCM_DisableBackupReset(void);

void RCM_EnableAHBPeriphClock(uint32_t AHBPeriph);
void RCM_DisableAHBPeriphClock(uint32_t AHBPeriph);
void RCM_EnableAPB2PeriphClock(uint32_t APB2Periph);
void RCM_DisableAPB2PeriphClock(uint32_t APB2Periph);
void RCM_EnableAPB1PeriphClock(uint32_t APB1Periph);
void RCM_DisableAPB1PeriphClock(uint32_t APB1Periph);

void RCM_EnableAHBPeriphReset(uint32_t AHBPeriph);
void RCM_DisableAHBPeriphReset(uint32_t AHBPeriph);
void RCM_EnableAPB1PeriphReset(uint32_t APB1Periph);
void RCM_DisableAPB1PeriphReset(uint32_t APB1Periph);
void RCM_EnableAPB2PeriphReset(uint32_t APB2Periph);
void RCM_DisableAPB2PeriphReset(uint32_t APB2Periph);

void RCM_EnableIntFlag(uint8_t interrupt);
void RCM_DisableIntFlag(uint8_t interrupt);
uint16_t RCM_ReadStatusFlag(RCM_FLAG_T flag);
void RCM_ClearStatusFlag(void);
uint8_t RCM_ReadIntFlag(RCM_INT_T interrupt);
void RCM_ClearIntFlag(uint8_t interrupt);

/**@} end of group RCM_Fuctions*/
/**@} end of group RCM_Driver*/
/**@} end of group Peripherals_Library*/

#ifdef __cplusplus
}
#endif

#endif /* __RCM_H */
